#!/usr/bin/perl
#
# Copyright (c) Ensim Corporation, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#

use Fcntl;
use File::Copy;
use Getopt::Std;
%options = ();

#default values
$httpd_bin='/usr/sbin/httpd';
$httpd_conf='/etc/httpd/conf/httpd.conf';

#Get cmd ln opts
getopts("bc:n:l:s:", \%options);

if ( $options{c} ) {
	$httpd_conf=$options{c};
}

if ( $options{b} ) {
	$back_up=1;
} else {
	$back_up=0;
}

if ( $options{n} ) {
	$mod_name=$options{n};
} else {
	usage();
	exit 1;
}

if ( $options{l} ) {
	$lib_name=$options{l};
} else {
	usage();
	exit 1;
}

if ( $options{s} ) {
	$mod_source=$options{s};
} else {
	usage();
	exit 1;
}
if ( $ARGV[$#ARGV] =~ /(enable)|(disable)|(delete)/ ) {
	$operation=$ARGV[$#ARGV];
} else {
	usage();
	exit 1;
}

sub usage {
	print "Usage: $0 [options] <params> <action>\n";
	print "options:  -c <httpd-conf-file>\n";
	print "          -b create back up of current conf file\n";
	print "params:   -n <module-name>\n";
	print "          -l <library-name>\n";
	print "          -s <module-source>\n";
	print "actions:  enable\n";
	print "          disable\n";
	print "          delete\n";
}


# check_conf( <path-to-httpd>, <path-to-httpd-conf> )
sub check_conf {
	my ($httpd_bin, $httpd_conf) = @_;
	my $output;

	$output=`$httpd_bin -f $httpd_conf -T 2>&1`;
	if ( $? == 0 && $output =~ /OK/ ) {
		return;
	} else {
		die "$output";
	}
}

# read_conf( <path-to-httpd-conf> )
# returns reference to the array
sub read_conf {
	my ($httpd_conf) = $_[0];
	if ( -f $httpd_conf ) {
		open(SOURCE, "< $httpd_conf") 
			or die "Couldn't open $httpd_conf for reading: $!\n";
		@file_array=<SOURCE>;
		close(SOURCE);
	} else {
		die 'File not found';
	}
	
	return \@file_array;
}

sub write_conf {
	my $array_ref = $_[0];
	my $file_path="/tmp/httpd.conf.$$";

	sysopen(OUT, $file_path, O_WRONLY|O_EXCL|O_CREAT, 0644) or die $!;
	print OUT @$array_ref;
	close(OUT);
	return "$file_path";
}


# Takes the reference to the conf array.
# returns array index of last LoadModule found
sub last_load {
	my $array_ref = $_[0];
	my $curr_line;
	my $i;
	my $last_idx;
	my $line_cnt = $#{@$array_ref};

	for ( $i = 0; $i <= $line_cnt; $i++ ) {
		$curr_line=$array_ref->[$i];
		if ( $curr_line=~ /^#*LoadModule/ ) {
			$last_idx=$i;
		}
	}

	return $last_idx;
}

# Takes the reference to the conf array.
# Returns array index of last AddModule found
sub last_add {
        my $array_ref = $_[0];
        my $curr_line;
        my $i;
        my $last_idx;
        my $line_cnt = $#{@$array_ref};
 
        for ( $i = 0; $i <= $line_cnt; $i++ ) {
                $curr_line=$array_ref->[$i];
                if ( $curr_line=~ /^#*AddModule/ ) {
                        $last_idx=$i;
                }
        }
 
        return $last_idx;
}

sub insert_line {
	my $array_ref = $_[0];
	my $array_pos = $_[1];
	my $insert_str = $_[2];
	chomp $insert_str;

	splice(@$array_ref, $array_pos , 0, "$insert_str\n"); 
}

sub delete_line {
	my $array_ref = $_[0];
	my $array_pos = $_[1];

	splice(@$array_ref, $array_pos, 1);
}

sub find_load {
	my $array_ref = $_[0];
	my $mod_name = $_[1];
	my $lib_name = $_[2];
	my $count=0;
	my $line;
	my $i;
	my $last_idx;
	my $line_cnt = $#{@$array_ref};

	for ( $i = 0; $i <= $line_cnt; $i++ ) {
		$curr_line=$array_ref->[$i];
		if ( $curr_line=~ /^#*LoadModule\s+$mod_name\s+modules\/$lib_name/ ) {
			++$count;
			$line=$i;
		}
	}
	if ( $count != 1  ) {
		return -1;
	} else {
		return $line;
	}
}

sub find_add {
        my $array_ref = $_[0];
        my $mod_source = $_[1];
        my $count=0;
        my $line;
        my $i;
        my $last_idx;
        my $line_cnt = $#{@$array_ref};
 
        for ( $i = 0; $i <= $line_cnt; $i++ ) {
                $curr_line=$array_ref->[$i];
                if ( $curr_line=~ /^#*AddModule\s+$mod_source/ ) {
                        ++$count;
                        $line=$i;
                }
        }
        if ( $count != 1  ) {
                return -1;
        } else {
                return $line;
        }
}

sub comment_line {
	my $array_ref = $_[0];
	my $array_idx = $_[1];
	my $curr_line;

	$curr_line=$array_ref->[$array_idx];
	if ( $curr_line =~ /^#/ ) {
		return 0;
	} else {
		$curr_line="#$curr_line";
		$array_ref->[$array_idx]=$curr_line;
		return 0;
	}
}

sub uncomment_line {
        my $array_ref = $_[0];
        my $array_idx = $_[1];
        my $curr_line;
 
        $curr_line=$array_ref->[$array_idx];
        if ( ! $curr_line =~ /^#/ ) {
                return 0;
        } else {
                $curr_line =~ s/^#// ;
                $array_ref->[$array_idx]=$curr_line;
                return 0;
        }
}

check_conf($httpd_bin, $httpd_conf);
$array_ref=read_conf($httpd_conf);

if ( $operation eq 'enable' ) {
	$load_location=find_load($array_ref, $mod_name, $lib_name);
	if ( $load_location == -1 ) {
		$myString="LoadModule $mod_name		modules/$lib_name";
		$last_load_idx=last_load($array_ref);
		++$last_load_idx;
		insert_line($array_ref, $last_load_idx, $myString);
	} else {
		uncomment_line($array_ref, $load_location);
	}

	$add_location=find_add($array_ref, $mod_source);
	if ( $add_location == -1 ) {
		$myString="AddModule $mod_source";
		$last_add_idx=last_add($array_ref);
		++$last_add_idx;
		insert_line($array_ref, $last_add_idx, $myString);
	} else {
		uncomment_line($array_ref, $add_location);
	}
} elsif ( $operation eq 'disable' ) {
	$load_location=find_load($array_ref, $mod_name, $lib_name);
	if ( $load_location != '-1' ) {
		comment_line($array_ref, $load_location);
	}

	$add_location=find_add($array_ref, $mod_source);
	if ( $add_locatioin != '-1' ) {
		comment_line($array_ref, $add_location);
	}
} elsif ( $operation eq 'delete' ) {
	$load_location=find_load($array_ref, $mod_name, $lib_name);
	if ( $load_location != -1 ) {
		delete_line($array_ref, $load_location);
	}
	$add_location=find_add($array_ref, $mod_source);
	if ( $add_location != -1 ) {
		delete_line($array_ref, $add_location);
	}
}

$tmp_conf=write_conf($array_ref);
eval {
	check_conf($httpd_bin, $tmp_conf);
};
$err_msg=$@;

if ( $err_msg ) {
	unlink($tmp_conf)
		or die "Unable to remove $tmp_conf: $!\n";
	print STDERR "$err_msg";
	exit 1;
} else {
	if ( $back_up == 1 ) {
		copy($httpd_conf, '/tmp/httpd.conf-backup')
			or die "Unable to backup $httpd_conf: $!\n";
	}
	unlink($httpd_conf)
		or die "Unable to remove $httpd_conf:$!\n";
	move($tmp_conf, $httpd_conf) 
		or die "Unable to move $tmp_conf to $httpd_conf: $!\n";	
}

exit 0;
